'use strict';

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * 单行筛选项
 */
var React = require('react');
var _ = require('lodash');
var classNames = require('classnames');

var _require = require('yylib-ui'),
    YYClass = _require.YYClass,
    YYCol = _require.YYCol,
    YYRow = _require.YYRow,
    YYButton = _require.YYButton,
    YYMessage = _require.YYMessage;

var AuthToken = require('yylib-utils/AuthToken');
var Utils = require('./Utils');

var PropTypes = YYClass.PropTypes;
var field = YYClass.field;

var YYFilterMultiple = require('./CPFilterMultiple');
var YYFilterSingle = require('./CPFilterSingle');
var YYFilterToggle = require('../../../searchfilter/YYFilterToggle');
var YYFilterSelected = require('../../../searchfilter/YYFilterSelected');

require('../../../searchfilter/index.css');

//定义META
var uiMeta = {
    name: 'searchitem',
    fields: [
    //field参数列表：字段属性名|属性默认值|属性类型|是否必填|属性描述
    field('title', '', PropTypes.string, true, '条件标题'), field('splitText', '', PropTypes.string, true, '标题后的分割符，可以为空字符串'), field('rowKey', '', PropTypes.string, true, '条件行的键值名称'), field('items', [], PropTypes.array, true, '[{value:"id1", text:"name1",disabled:true,type:"text"},{value:"id2", text:"name2",disabled:false,type:"date"},{value:"id3", text:name3",disabled:false,,type:"input"}]'), field('multiple', true, PropTypes.bool, true, '是否启用多选模式'), field('onClick', null, PropTypes.func, false, '点击单选时的回调函数,接收参数为该项数据对象{ value, checked}'), field('onSubmit', null, PropTypes.func, false, '多选时,点击确定时的回调函数，接收参数为选中项数组[{value:"id1", text:"name1"}]'), field('onCancel', null, PropTypes.func, false, '多选时,点击取消时的回调函数'), field('minCount', 4, PropTypes.number, false, '当数据大于4条时显示更多按钮'), field('isSetting', false, PropTypes.bool, true, '是否启用设置按钮'), field('onSettingBtnClick', null, PropTypes.func, false, '点击设置按钮的回调'), field('extra', '', PropTypes.element, false, '右侧操作区域'), field('defaultSelectedKeys', [], PropTypes.array, false, '多选状态下默认选中的keys'), field('defaultActivedKey', '', PropTypes.array, false, '单选状态下默认选中的key'), field('colSpan', 6, PropTypes.number, false, '每个单位所占比例，24等份')]
};

var CPFilterRow = YYClass.create({
    uiMeta: uiMeta,
    getInitialState: function getInitialState() {
        var _props = this.props,
            items = _props.items,
            defaultSelectedKeys = _props.defaultSelectedKeys,
            defaultActivedKey = _props.defaultActivedKey;

        var selectedItems = _.filter(items, function (item) {
            return defaultSelectedKeys.indexOf(item.value) !== -1;
        });

        return {
            selectedItems: selectedItems, //多选状态下选中项
            defaultSelectedKeys: defaultSelectedKeys,
            isMultiple: false, //是否为多选态
            isShowAll: false, //是否显示全部数据
            defaultActivedKey: defaultActivedKey,
            itemArr: []
        };
    },
    componentDidMount: function componentDidMount() {
        var _this = this;

        var _props2 = this.props,
            code = _props2.code,
            refId = _props2.refId;

        if (code || refId) {
            this.requestData(this.props, function (items) {
                _this.setState({ itemArr: items });
            });
        }
    },

    componentWillReceiveProps: function componentWillReceiveProps(nextProps, preProps) {
        var _this2 = this;

        if ("defaultActivedKey" in nextProps && (nextProps.defaultActivedKey !== this.props.defaultActivedKey || nextProps.defaultActivedKey == '')) {
            this.setState({
                defaultActivedKey: nextProps.defaultActivedKey
            });
        }
        if ("defaultSelectedKeys" in nextProps && "items" in nextProps) {
            var selectedItems = _.filter(nextProps.items, function (item) {
                return nextProps.defaultSelectedKeys.indexOf(item.value) !== -1;
            });
            this.setState({
                selectedItems: selectedItems, //多选状态下选中项
                defaultSelectedKeys: nextProps.defaultSelectedKeys
            });
        }
        if ("code" in nextProps && nextProps.code !== this.props.code || "refId" in nextProps && nextProps.refId !== this.props.refId) {
            this.requestData(nextProps, function (items) {
                _this2.setState({ itemArr: items });
            });
        }
    },

    //多选|单选切换
    _toggleMultiple: function _toggleMultiple() {
        this.setState({ isMultiple: !this.state.isMultiple });
    },
    //切换更多|收起
    _toggleShowAll: function _toggleShowAll(isExpand) {
        if (this.state.isMultiple) {
            //多选态
            this.setState({ isShowAll: isExpand, isMultiple: false });
        } else {
            //单选态
            this.setState({ isShowAll: isExpand });
        }
    },
    //单选－提交
    _submitSigle: function _submitSigle(item, checked, rowKey) {
        if (this.props.onClick) {
            this.props.onClick(item, checked, rowKey);
        }
        if (checked) {
            this.setState({ checkedKey: item.value });
        } else {
            this.setState({ checkedKey: '' });
        }
    },
    //多选态-提交
    _submitMultiple: function _submitMultiple(items, rowKey) {
        var defaultSelectedKeys = _.map(items.value, function (item) {
            return item;
        });
        this.setState({
            isMultiple: false,
            selectedItems: items,
            defaultSelectedKeys: defaultSelectedKeys
        });
        if (this.props.onSubmit) {
            this.props.onSubmit(items, rowKey);
        }
    },
    //多选态-移除
    _removeMultiple: function _removeMultiple(rowKey) {
        this.setState({
            selectedItems: [],
            defaultSelectedKeys: []
        });
        if (this.props.onSubmit) {
            this.props.onSubmit([], rowKey);
        }
    },
    //多选态-取消
    _cancelMultiple: function _cancelMultiple(rowKey) {
        this.setState({ isMultiple: false });
        if (this.props.onCancel) {
            this.props.onCancel(rowKey);
        }
    },
    _onSettingBtnClick: function _onSettingBtnClick() {
        if (this.props.onSettingBtnClick) {
            this.props.onSettingBtnClick();
        }
    },
    _createSettingBtn: function _createSettingBtn(isSetting) {
        if (isSetting) {
            return React.createElement(
                YYButton,
                { className: 'yyui-filter-btn-multiple', onClick: this._onSettingBtnClick },
                '\u8BBE\u7F6E'
            );
        }
        return null;
    },
    //创建多选按钮
    _createMultiBtn: function _createMultiBtn(isMultiple) {
        if (this.props.multiple && !isMultiple) {
            return React.createElement(
                YYButton,
                { className: 'yyui-filter-btn-multiple', onClick: this._toggleMultiple },
                '\u591A\u9009'
            );
        }
        return null;
    },
    //创建更多按钮
    _createMoreBtn: function _createMoreBtn(isMultiple) {
        var _state = this.state,
            isShowAll = _state.isShowAll,
            itemArr = _state.itemArr;
        var _props3 = this.props,
            items = _props3.items,
            minCount = _props3.minCount;


        if (items.length > minCount || itemArr.length > minCount) {
            return React.createElement(YYFilterToggle, {
                expandText: '\u6536\u8D77',
                collapseText: '\u66F4\u591A',
                expandIcon: 'up',
                collapseIcon: 'down',
                newMore: true,
                expand: isMultiple ? true : isShowAll,
                onExpandOrCollapse: this._toggleShowAll

            });
        }
        return null;
    },
    //创建内容
    _createContent: function _createContent(isMultiple) {
        //启用了多选模式才使用多选组件
        var _state2 = this.state,
            selectedItems = _state2.selectedItems,
            defaultSelectedKeys = _state2.defaultSelectedKeys,
            defaultActivedKey = _state2.defaultActivedKey,
            itemArr = _state2.itemArr;
        var _props4 = this.props,
            multiple = _props4.multiple,
            rowKey = _props4.rowKey,
            items = _props4.items,
            onClearFilter = _props4.onClearFilter,
            minCount = _props4.minCount,
            code = _props4.code;


        if (multiple && !isMultiple && selectedItems.length) {
            return React.createElement(YYFilterSelected, {
                rowKey: rowKey,
                selectedItems: selectedItems,
                onRmoveItems: this._removeMultiple
            });
        }
        if (isMultiple) {
            //多选态
            return React.createElement(YYFilterMultiple, {
                rowKey: rowKey,
                items: code ? itemArr : items,
                colSpan: this.props.colSpan,
                onSubmit: this._submitMultiple,
                onCancel: this._cancelMultiple,
                defaultSelectedKeys: defaultSelectedKeys
            });
        } else {
            //单选态
            return React.createElement(YYFilterSingle, {
                rowKey: rowKey,
                items: code ? itemArr : items,
                minCount: minCount,
                colSpan: this.props.colSpan,
                onClick: this._submitSigle,
                showAll: this.state.isShowAll,
                onClearFilter: onClearFilter,
                defaultActivedKey: defaultActivedKey,
                disabled: this.props.disabled
            });
        }
    },
    requestData: function requestData(props, callback) {
        var code = props.code,
            dataparams = props.dataparams,
            serverUrl = props.serverUrl,
            enableOrg = props.enableOrg,
            orgId = props.orgId,
            refId = props.refId,
            sysType = props.sysType;

        var _orgId = null;
        if (enableOrg) {
            _orgId = AuthToken.getOrgaId();
        }
        if (orgId) {
            _orgId = orgId;
        }
        if (sysType === 'ncc') {
            Utils.defdocByCode(serverUrl, refId, function (result) {
                if (result) {
                    if (result.success) {
                        var data = result.data.content || [];
                        var itemArr = [];
                        data.map(function (row) {
                            itemArr.push({ compare: 'eq', type: 'input', text: row.refname, value: row.refpk });
                        });
                        callback(itemArr);
                    } else {
                        YYMessage.error(result.backMsg);
                    }
                }
            });
        } else {
            Utils.findDefDocListByCode(serverUrl, code, _orgId, dataparams, function (result) {
                if (result) {
                    if (result.success) {
                        var data = _.map(result.backData, function (item) {
                            return { compare: 'eq', type: 'input', text: item.name, value: item.code };
                        });
                        callback(data);
                    } else {
                        YYMessage.error(result.backMsg);
                    }
                }
            });
        }
    },

    render: function render() {
        var _props5 = this.props,
            multiple = _props5.multiple,
            title = _props5.title,
            splitText = _props5.splitText,
            extra = _props5.extra,
            isSetting = _props5.isSetting,
            style = _props5.style;
        var isMultiple = this.state.isMultiple;

        var extraSpan = extra ? 6 : 0;
        //启用了多选模式才使用多选组件
        isMultiple = multiple && isMultiple;
        var multipleClass = classNames(_defineProperty({}, ' multiple', isMultiple));
        return React.createElement(
            YYRow,
            { className: "yyui-filter-line" + multipleClass, style: style },
            React.createElement(
                YYCol,
                { span: 2 },
                React.createElement(
                    'span',
                    null,
                    '' + (title + splitText)
                )
            ),
            React.createElement(
                YYCol,
                { span: 15 - extraSpan },
                this._createContent(isMultiple)
            ),
            React.createElement(
                YYCol,
                { span: 7, className: this.className() + '-rightAction' },
                this._createMoreBtn(isMultiple),
                this._createMultiBtn(isMultiple),
                this._createSettingBtn(isSetting)
            ),
            extra ? React.createElement(
                YYCol,
                { span: extraSpan },
                extra
            ) : null
        );
    }
});

module.exports = CPFilterRow;