"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

/**
 * crypto-js 方式实现的加密解密
 */
var CryptoJS = require("./crypto-js");
var Base64 = CryptoJS.enc.Base64;
var Utf8 = CryptoJS.enc.Utf8;
var Hex = CryptoJS.enc.Hex;

/**
 * 增强加密
 */
var KEY_ENHANCE = Base64.parse("ZGIyMTM5NTYxYzlmZTA2OA==");
/**
 * 关键字
 */
var PRIVATE_KEY = generateKey(uuidv4());
/**
 * 是否进行加密解密
 * 可变数据
 */
var CipherFlag = false;

/**
 * 采用hex方式处理
 */
var isHex = false;

/**
 * 加密解密参数
 */
var cfg = {
    iv: KEY_ENHANCE,
    mode: CryptoJS.mode.CBC,
    padding: CryptoJS.pad.Pkcs7
};

/**
 * 生成唯一标识
 */
function uuidv4() {
    return "xxxxxxxxxxxx4xxxyxxxxxxxxxxxxxxx".replace(/[xy]/g, function (c) {
        var r = Math.random() * 16 | 0,
            v = c == "x" ? r : r & 0x3 | 0x8;
        return v.toString(16);
    });
}

/**
 * 生成 秘钥的方法
 * @param {*} text
 * @param {*} bytes
 */
function generateKey() {
    var text = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : "";

    return Base64.stringify(Utf8.parse(text)) || "ZGIyMTM5NTYxYzlmZTA2OA==";
}

/**
 * 加密
 * @param {*} message  明文信息
 * @param {*} key 秘钥
 * @param {*} standard 标准
 */
function encrypt() {
    var message = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : "";
    var key = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : PRIVATE_KEY;
    var standard = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : "AES";

    var keyDigest = Base64.parse(key);
    var encrypted = CryptoJS[standard].encrypt(message, keyDigest, cfg);
    // 这里需要注意  先拿结果  然后再做base64 加密  encrypted.ciphertext 是个16进制字符串
    var ciphertext = Base64.stringify((isHex ? Hex : Utf8).parse(encrypted.ciphertext.toString()));
    // let ciphertext = Base64.stringify(Hex.parse(encrypted.ciphertext.toString()));
    return ciphertext;
}

/**
 * 解密
 * @param {*} message  密文信息
 * @param {*} key 秘钥
 * @param {*} standard 标准
 */
function decrypt() {
    var message = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : "";
    var key = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : PRIVATE_KEY;
    var standard = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : "AES";

    var keyDigest = Base64.parse(key);
    //  base64 解密  plaintext 只能是Hex编码的base64字节数组
    var plaintext = isHex ? message : Base64.stringify(Hex.parse(Base64.parse(message).toString(Utf8)));
    var decrypted = CryptoJS[standard].decrypt(plaintext, keyDigest, cfg);
    return decrypted.toString(Utf8);
}

// JSON 工具类二次封装
var Json = {
    toJSON: function toJSON(o) {
        return JSON.stringify(o);
    },
    parseJSON: function parseJSON(s) {
        return JSON.parse(s);
    }
};

/**
 * 对json进行加密
 * @param {*} json
 * @param {*} decryptOpt
 */
function jsonToEncrypt(json) {
    var decryptOpt = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    var ciphertext = void 0;
    try {
        ciphertext = encrypt.apply(undefined, [Json.toJSON(json)].concat(_toConsumableArray(decryptOpt)));
    } catch (error) {
        ciphertext = json;
        console.log("encrypt error: %s", error);
    }
    return ciphertext;
}

var methods = {
    encrypt: encrypt,
    decrypt: decrypt,
    jsonToEncrypt: jsonToEncrypt
};

var Cipher = _extends({
    Json: Json,
    PRIVATE_KEY: PRIVATE_KEY,
    CipherFlag: CipherFlag
}, function () {
    // 对当前组件中的方法进行处理  主要是到处添加 开关太麻烦
    var defaultFunction = function defaultFunction(data) {
        return data;
    };

    var _loop = function _loop(k) {
        //methods[k] = !CipherFlag ? defaultFunction : methods[k];
        var f = methods[k];
        methods[k] = function () {
            console.time("" + k);
            // 包装一次传递参数
            var res = (!Cipher.CipherFlag ? defaultFunction : f).apply(this, arguments);
            console.timeEnd("" + k);
            return res;
        };
    };

    for (var k in methods) {
        _loop(k);
    }
    return methods;
}());

// console.log(Cipher);

exports.default = Cipher;

/**
 * 不透明的加密
 * @param {*} msg
 */

var opaqueEncrypt = function opaqueEncrypt(msg) {
    if (!msg) {
        console.log('加密没有原文');
        return '';
    }
    return encrypt(msg, '4fa8959db7b4423a99f056e299914128');
};
/**
 * 不透明的解密
 * @param {*} msg
 */
var opaqueDecrypt = function opaqueDecrypt(msg) {
    if (!msg) {
        console.log('解密没有原文');
        return '';
    }
    return decrypt(msg, '4fa8959db7b4423a99f056e299914128');
};

// 抛出一系列方法
exports.encrypt = encrypt;
exports.decrypt = decrypt;
exports.jsonToEncrypt = jsonToEncrypt;
exports.opaqueEncrypt = opaqueEncrypt;
exports.opaqueDecrypt = opaqueDecrypt;