'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.openTo = openTo;
exports.getRealAppCode = getRealAppCode;

var _ajax = require('./ajax');

var _ajax2 = _interopRequireDefault(_ajax);

var _jsCookie = require('js-cookie');

var _jsCookie2 = _interopRequireDefault(_jsCookie);

var _yylibUi = require('yylib-ui');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function randomNumber() {
    var now = new Date();
    var month = now.getMonth() + 1;
    var day = now.getDate();
    var hour = now.getHours();
    var minutes = now.getMinutes();
    var seconds = now.getSeconds();
    return now.getFullYear().toString() + month.toString() + day + hour + minutes + seconds + Math.round(Math.random() * 23 + 100).toString();
}

function setAppUrl(key, newParams) {
    var session = sessionStorage.getItem("NCCAPPURL");
    session = session && JSON.parse(session);
    if (session) {
        session[key] = newParams;
    } else {
        session = _defineProperty({}, key, newParams);
    }
    session = JSON.stringify(session);
    sessionStorage.setItem("NCCAPPURL", session);
}

function openApp(param, callback) {
    var appcode = param.appcode,
        pagecode = param.pagecode,
        serverUrl = param.serverUrl;

    var queryUrl = serverUrl + "/nccloud/platform/appregister/openapp.do";
    var params = {
        "busiParamJson": JSON.stringify({ appcode: appcode, pagecode: pagecode }),
        "sysParamJson": { pagecs: _jsCookie2.default.get("cookiets"), appcode: '10228888' }
    };

    _ajax2.default.postJSONWithCookies(queryUrl, params, function (result) {
        if (result && result.success) {
            callback && callback(result.data);
        } else {
            _yylibUi.YYMessage.error("打开节点失败或者没有该应用的权限");
        }
    }, function () {
        _yylibUi.YYMessage.error("打开节点失败或者没有该应用的权限");
    });
}

/**
 *
 * @param url 需要跳转到的页面地址，如果不知道可以去应用注册里面查看
 * @param params appcode：应用编码 必要参数
 *               pagecode：页面编码 必要参数
 *               serverUrl：请求地址,
 *               其他参数可自定义
 *               参数格式key-value {appcode:'123',pagecode:'234',...}
 */
function openTo(url, params) {

    if (!params.appcode || !params.pagecode) {
        _yylibUi.YYMessage.error("缺少必要参数appcode,pagecode");
        return;
    }
    //解析参数，拼接
    Object.keys(params).forEach(function (key, index) {
        url = url + (index === 0 ? "?" : "&") + key + "=" + params[key];
    });
    url = urlEncode(url);

    params.serverUrl = params.serverUrl || window.EnvConfig.ADDR;
    //请求后台，获取面包屑以及鉴权
    openApp(params, function (data) {
        var appId = encodeURIComponent(encodeURIComponent(data.appid));
        var field = encodeURIComponent(encodeURIComponent(data.field));
        var module = encodeURIComponent(encodeURIComponent(data.module));
        var menuclass = encodeURIComponent(encodeURIComponent(data.menuclass));
        var menu = encodeURIComponent(encodeURIComponent(data.menu));

        //生成导航以及面包屑
        url = url + "&c=" + params.appcode + "&p=" + params.pagecode + "&ar=" + appId + "&n=" + menu + "&b1=" + field + "&b2=" + module + "&b3=" + menuclass;
        var windowTabId = randomNumber();
        setAppUrl(windowTabId, url);
        window.open('/nccloud/resources/workbench/public/common/main/index.html?type=jz#/ifr?page=' + windowTabId);
    });
}

function urlEncode(url) {
    console.log("ifr=" + encodeURIComponent(encodeURIComponent(url)));
    return "ifr=" + encodeURIComponent(encodeURIComponent(url));
}

/**
 * 获取AppCode
 */
function getRealAppCode() {
    if (!window.parent || !window.parent.location) {
        return undefined;
    }
    var appN = window.parent.location.hash.split('?');
    appN = appN && appN[1];
    appN = appN && appN.split('=')[1];

    if (!window.top) {
        return undefined;
    }
    var nccAppUrl = window.top.sessionStorage.NCCAPPURL;

    nccAppUrl = nccAppUrl && JSON.parse(nccAppUrl);

    nccAppUrl = nccAppUrl && nccAppUrl[appN];

    nccAppUrl = nccAppUrl && decodeURIComponent(decodeURIComponent(nccAppUrl));

    var params = nccAppUrl && nccAppUrl.split("&");

    var app_code = '';
    if (params) {
        params.forEach(function (item) {
            if (item.indexOf('=') !== -1 && item.split('=') && item.split('=') instanceof Array) {
                if (item.split('=')[0] === 'c') {
                    if (item.split('=')[1]) {
                        app_code = decodeURIComponent(decodeURIComponent(item.split('=')[1]));
                    }
                }
            }
        });
        return app_code;
    }

    return undefined;
}